import os
from pymongo import MongoClient
import time

MONGO_HOST = os.environ.get('CRAWLAB_MONGO_HOST')
MONGO_PORT = int(os.environ.get('CRAWLAB_MONGO_PORT', '27017'))
MONGO_DB = os.environ.get('CRAWLAB_MONGO_DB')
MONGO_USERNAME = os.environ.get('CRAWLAB_MONGO_USERNAME')
MONGO_PASSWORD = os.environ.get('CRAWLAB_MONGO_PASSWORD')
MONGO_AUTHSOURCE = os.environ.get('CRAWLAB_MONGO_AUTHSOURCE')

mongo_client = MongoClient(
    host=MONGO_HOST,
    port=MONGO_PORT,
    username=MONGO_USERNAME,
    password=MONGO_PASSWORD,
    authSource=MONGO_AUTHSOURCE,
) if MONGO_HOST else None

mongo_db = mongo_client.get_database(MONGO_DB) if mongo_client else None

refresh_time = {}


def refresh_session(session):
    id = session.session_id.get('id')
    last_refresh_time = refresh_time.get(id)
    if last_refresh_time is None or time.time() - last_refresh_time < 240:
        print(f'last refresh time too short, ignore refresh request ')
        return
    mongo_db.command("refreshSessions", [{'id': id}])
    print(f'refresh session success: {id}')
    refresh_time[id] = time.time()



from scrapy.exceptions import DropItem
import requests
import os
import json
import time

class FlowPipeline(object):

    def process_item(self, item, spider):
        unique_id = item.get('unique_id')
        if not unique_id:
            raise DropItem('unique_id is None')

        data = item.deepcopy().to_dict()

        crawled_at = data.get('crawled_at')
        if crawled_at:
            data['crawled_at'] = data['crawled_at'].isoformat()

        # convert datetime to str before serialize
        flow_urls = item.get('flow_urls', None)
        if not flow_urls:
            flow_urls = spider.settings.get('FLOW_URLS')
        if flow_urls:
            data_helper_url = spider.settings.get('DATA_HELPER_URL')
            if not data_helper_url:
                data_helper_url = os.environ.get('DATA_HELPER_URL')
            if not data_helper_url:
                spider.logger.error(f'data helper url not found, config url to settings')
                return item
            else:
                config_type = type(flow_urls)
                flows = []
                if config_type == str:
                    # 本地开发直接发送 flow
                    if flow_urls.startswith('http://127.0.0.1') or flow_urls.startswith('http://localhost') :
                        response = requests.post(flow_urls, json=data, timeout=10)
                        if int(response.status_code) == 200:
                            spider.logger.info(f'req addr: {flow_urls}  {unique_id} request success')
                        else:
                            spider.logger.info(f'req addr: {flow_urls} {unique_id} request fail, {response.status_code} {data}')
                        return item
                    flows.append(flow_urls)
                elif config_type == list:
                    flows = flow_urls
                else:
                    spider.logger.error(f'flow urls config type error, make sure the type must be an array or string')
                    return item
                for i in range(10):
                    try:
                        response = requests.post(data_helper_url, json={
                            'flow_urls': flows,
                            'payload': json.dumps(data),
                        }, timeout=5)
                        if response.status_code == 200:
                            spider.logger.info(f'send data to helper success: {item.get("unique_id")} {data_helper_url} {flows}')
                            return item
                        else:
                            spider.logger.warn(f'send data to helper fail: {item.get("unique_id")} {data_helper_url} {flows} {response.status_code}')
                    except:
                        spider.logger.warn(f'send data to helper fail, try times: {i} : {item.get("unique_id")} {data_helper_url} {flows}')
                    time.sleep(3)
        return item




from scrapy_nc.db import mongo_db
import json
from scrapy.exceptions import DropItem


class MongoPipeline(object):
    def process_item(self, item, spider):
        my_item = item.deepcopy()
        item_dict = dict(my_item)
        unique_id = item['unique_id']
        if not unique_id:
            raise DropItem('unique_id is None')

        crawled_at = item['crawled_at']
        if not crawled_at:
            raise DropItem('crawled_at is None')

        spider.collection.update_one({
            'unique_id':   my_item.get('unique_id')
        }, {"$set": item_dict}, upsert=True)
        return item

    def open_spider(self, spider):
        pass

    @classmethod
    def from_crawler(cls, crawler):
        spider = crawler.spider
        instance = cls()
        if not mongo_db:
            spider.logger.error('mongodb no configuration')
            return instance
        spider.collection = mongo_db.get_collection(spider.name)
        res = json.dumps(spider.collection.index_information())
        spider.logger.info(
            f'index_information {res}')
        index_name = 'unique_id'
        if index_name not in spider.collection.index_information():
            spider.collection.create_index(
                'unique_id', unique=True, name=index_name)
            spider.logger.info(f"create unique index {index_name}")
        ttl = spider.settings.get('DATA_TTL')
        if ttl is None:
            ttl = 86400 * 30 * 3 # 默认过期时间 90 天
            spider.logger.info(f'not found data_ttl,  set ttl 30 days')
        if ttl == -1:
            return instance
        if ttl > 0:
            expire_index_name = "crawled_at"
            if expire_index_name not in spider.collection.index_information():
                spider.collection.create_index(
                    "crawled_at", name=expire_index_name, expireAfterSeconds=ttl,
                )
                spider.logger.info(f'create ttl index {expire_index_name}, ttl: {ttl}')
        return instance

